/** A simple player that makes a little fort and hides behind it. Feel free to use this as a
starting point for your own player.
ICPC Challenge
Sturgill, Baylor University */

# include "Const.h"
# include <vector>
# include <string>
# include <iostream>
# include <cstdlib>
# include <cstdio>

using namespace std;

struct Child													// Simple representation for a child in the Game
{
	int x, y;													// Location of the child
	bool standing;												// True if the child is standing
	int color;												// Side the child is on
	int holding;												// What's the child holding
	int dazed;												// How many more turns this child is dazed
};

int dispX[] = {1, 0, -1, 0};										// X displacement for each walk direction
int dispY[] = {0, 1, 0, -1};										// Y displacement for each walk direction
const int GROUND_CHILD = 10;										// Extra constant to mark child locations on the map
const int GROUND_TAKEN = 11;										// Another extra constant to mark spaces we should not pick up from
const int FORT_SIZEX = 30;										// How big of a fort do we want to try to build
const int FORT_SIZEY = 30;
int walkDir[] = {1, 0, 2, 3};										// Walk direction for each child
int score[2];													// Current game score, for red and blue

vector< vector< int > > height (SIZE, vector< int > (SIZE));			// Current snow height in each cell
vector< vector< int > > ground (SIZE, vector< int > (SIZE));			// Contents of each cell
vector< Child > cList (CCOUNT * 2);								// List of current information about each child

bool chooseMove (int i, int turn)									// Try to pick a next move. Return false if the child just turns in place and chooses
{															// no move
	Child &child = cList[i];										// Figure out where this child is about to move

	if (child.dazed > 0)										// If we're dazed, we have to stay idle
	{
		cout << "idle" << endl;
		return true;
	}
	if (child.holding == HOLD_EMPTY)								// Pick up some snow if there's some nearby
	{
		int sx = -1, sy = -1;									// See if there's some snow nearby
		for (int ox = child.x - 1; ox <= child.x + 1; ox++)
		for (int oy = child.y - 1; oy <= child.y + 1; oy++)
		{
			// Is there snow to pick up? Can we pick it up? Do we want to?
			if (ox >= 0 && ox < FORT_SIZEX && oy >= 0 && oy < FORT_SIZEY && (ox != child.x || oy != child.y) && ground[ox][oy] == GROUND_EMPTY && height[ox][oy] > 0)
			{
				sx = ox;
				sy = oy;
			}
		}
		if (sx >= 0)											// Try to crouch down and pick up snow if there is some
		{
			if (child.standing)
			{
				cout << "crouch" << endl;
				return true;
			}
			/** Pick up snow from the space, and mark it as taken so another one of our children
			won't try to take it also */
			cout << "pickup " << sx << " " << sy << endl;
			ground[sx][sy] = GROUND_TAKEN;
			return true;
		}
	}
	if (child.holding == HOLD_P1)									// Crush one unit of powdered snow into a snowball
	{
		cout << "crush" << endl;
		return true;
	}
	if (child.holding == HOLD_S1)
	{
		if (!child.standing)									// We're holding a small snowball
		{
			cout << "stand" << endl;
			return true;
		}
		for (int j = CCOUNT; j < CCOUNT * 2; j++)					// See if we should throw a snowball at an opponent
		{
			if (cList[j].x >= 0)
			{
				int dx = cList[j].x - child.x;
				int dy = cList[j].y - child.y;
				int dsq = dx * dx + dy * dy;
				if (dsq < 8 * 8)								// If we see a nearby child, throw past them
				{
					cout << "throw " << child.x + dx * 2 << " " << child.y + dy * 2 << endl;
					return true;
				}
			}
		}
		int r = (walkDir[i] + 3) % 4;								// Is the wall to our right?
		int rightX = child.x + dispX[r];
		int rightY = child.y + dispY[r];

		if (rightX >= 0 && rightY >= 0 && (rightX == FORT_SIZEX || rightY == FORT_SIZEY) && height[rightX][rightY] < OBSTACLE_HEIGHT)
		{
			cout << "drop " << rightX << " " << rightY << endl;
			return true;
		}
	}

	int newX = child.x + dispX[walkDir[i]];							// See where the child is moving
	int newY = child.y + dispY[walkDir[i]];

	if (rand () % 10 == 0)										// Child may turn just because she feels like it
	{
		walkDir[i] = (walkDir[i] + 1) % 4;
		return false;
	}
	if (newX < 0 || newY < 0 || newX >= FORT_SIZEX || newY >= FORT_SIZEY)	// Child must turn if she hits the edge of the fort
	{
		walkDir[i] = (walkDir[i] + 1) % 4;
		return false;
	}
	// Turn must turn if the walk direction is blocked  
	if (ground[newX][newY] == GROUND_TREE || ground[newX][newY] == GROUND_CHILD || height[newX][newY] >= OBSTACLE_HEIGHT)
	{
		walkDir[i] = (walkDir[i] + 1) % 4;
		return false;
	}
	if (child.standing)											// Move on around the perimiter if we have nothing better to do
	{
		if (i == 0)
			cout << "run " << child.x + rand () % 3 << " " << child.y + rand () % 3 << endl;
		else
		{
			if (i == 1)
				cout << "run " << child.x + rand () % 3 << " " << child.y + rand () % 3 << endl;
			else
			{
				if (i == 2)
					cout << "run " << child.x + rand () % 3 << " " << child.y + rand () % 3 << endl;
				else
				{
					if (i == 3)
						cout << "run " << child.x + rand () % 3 << " " << child.y + rand () % 3 << endl;
				}
			}
		}
	}
	else
		cout << "crawl " << newX << " " << newY << endl;

	ground[child.x][child.y] = GROUND_TAKEN;						// Update the map, so that the next child won't try to move right into this one
	ground[newX][newY] = GROUND_CHILD;
  
	return true;
}

/** Main procedure */
int main ()
{
	int turnNum;

	cin >> turnNum;
	while (turnNum >= 0)
	{
		cin >> score[RED] >> score[BLUE];							// Read the current score
		for (int i = 0; i < SIZE; i++)							// Read the state of the field
		for (int j = 0; j < SIZE; j++)
		{
			char a, b;
			cin >> a >> b;
			if (a != '*')
			{
				height[i][j] = a - '0';
				ground[i][j] = b - 'a';
			}
			else
			{
				height[i][j] = -1;
				ground[i][j] = -1;
			}
		}

		string token;
		for (int i = 0; i < cList.size (); i++ )
		{
			cin >> token;
			if (token == "*")
			{
				cList[i].x = -1;								// Can't see this child
				cList[i].y = -1;
			}
			else
			{
				cList[i].x = atoi (token.c_str ());				// Can see this child, read the description
				cin >> cList[i].y;
				cList[i].color = i / CCOUNT;						// Fill in the child's side

				char ch;										// Read the child's stanse, what he's holding and how much longer he's dazed
				cin >> ch;
				cList[i].standing = (ch == 'S');
				cin >> ch;
				cList[i].holding = (ch - 'a');
				cin >> cList[i].dazed;
			}
		}

		for (int i = 0; i < cList.size (); i++)						// Mark child locations, so we can walk around them
		{
			if (cList[i].x >= 0)
				ground[cList[i].x][cList[i].y] = GROUND_CHILD;
		}

		for (int i = 0; i < CCOUNT; i++)							// Choose an action for each child
		{
			int tryCount = 0;									// Try up to three times to get a useful move

			while (tryCount < 3 && !chooseMove (i, turnNum))
				tryCount++;

			if (tryCount >= 3)									// If we can't think of a good move, wait and see if something happens
				cout << "idle" << endl;
		}
		cin >> turnNum;
	}
}
